@@ -9,9 +9,9 @@ import (
 	"github.com/onsi/ginkgo"
 	"github.com/onsi/gomega"
 
-	clusterv1client "github.com/open-cluster-management/api/client/cluster/clientset/versioned"
-	clusterv1 "github.com/open-cluster-management/api/cluster/v1"
-	clusterv1alpha1 "github.com/open-cluster-management/api/cluster/v1alpha1"
+	clusterv1client "open-cluster-management.io/api/client/cluster/clientset/versioned"
+	clusterv1 "open-cluster-management.io/api/cluster/v1"
+	clusterv1beta1 "open-cluster-management.io/api/cluster/v1beta1"
 
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
@@ -78,6 +78,8 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				managedCluster, err := clusterClient.ClusterV1().ManagedClusters().Get(context.TODO(), clusterName, metav1.GetOptions{})
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
 				gomega.Expect(managedCluster.Spec.LeaseDurationSeconds).To(gomega.Equal(int32(60)))
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should respond bad request when creating a managed cluster with invalid external server URLs", func() {
@@ -94,6 +96,8 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					admissionName,
 					invalidURL,
 				)))
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should forbid the request when creating an accepted managed cluster by unauthorized user", func() {
@@ -124,6 +128,9 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					saNamespace,
 					sa,
 				)))
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should accept the request when creating an accepted managed cluster by authorized user", func() {
@@ -149,19 +156,22 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				managedCluster := newManagedCluster(clusterName, true, validURL)
 				_, err = authorizedClient.ClusterV1().ManagedClusters().Create(context.TODO(), managedCluster, metav1.CreateOptions{})
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should accept the request when creating a managed cluster with clusterset specified by authorized user", func() {
 				clusterSetName := fmt.Sprintf("webhook-spoke-%s", rand.String(6))
 				ginkgo.By(fmt.Sprintf("create a managed cluster set %q", clusterSetName))
 
-				managedClusterSet := &clusterv1alpha1.ManagedClusterSet{
+				managedClusterSet := &clusterv1beta1.ManagedClusterSet{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: clusterSetName,
 					},
 				}
 
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				sa := fmt.Sprintf("webhook-sa-%s", rand.String(6))
@@ -189,19 +199,22 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				}
 				_, err = authorizedClient.ClusterV1().ManagedClusters().Create(context.TODO(), managedCluster, metav1.CreateOptions{})
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should forbid the request when creating a managed cluster with clusterset specified by unauthorized user", func() {
 				clusterSetName := fmt.Sprintf("webhook-spoke-%s", rand.String(6))
 				ginkgo.By(fmt.Sprintf("create a managed cluster set %q", clusterSetName))
 
-				managedClusterSet := &clusterv1alpha1.ManagedClusterSet{
+				managedClusterSet := &clusterv1beta1.ManagedClusterSet{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: clusterSetName,
 					},
 				}
 
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				sa := fmt.Sprintf("webhook-sa-%s", rand.String(6))
@@ -234,19 +247,27 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					sa,
 					clusterSetName,
 				)))
+
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 		})
 
 		ginkgo.Context("Updating a managed cluster", func() {
 			var clusterName string
-
 			ginkgo.BeforeEach(func() {
+				ginkgo.By(fmt.Sprintf("Creating managed cluster %q", clusterName))
 				clusterName = fmt.Sprintf("webhook-spoke-%s", rand.String(6))
 				managedCluster := newManagedCluster(clusterName, false, validURL)
 				_, err := clusterClient.ClusterV1().ManagedClusters().Create(context.TODO(), managedCluster, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 			})
 
+			ginkgo.AfterEach(func() {
+				ginkgo.By(fmt.Sprintf("Cleaning managed cluster %q", clusterName))
+				gomega.Expect(deleteManageClusterAndRelatedNamespace(clusterName)).ToNot(gomega.HaveOccurred())
+			})
+
 			ginkgo.It("Should not update the LeaseDurationSeconds to zero", func() {
 				ginkgo.By(fmt.Sprintf("try to update managed cluster %q LeaseDurationSeconds to zero", clusterName))
 				err := retry.RetryOnConflict(retry.DefaultRetry, func() error {
@@ -315,19 +336,21 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					saNamespace,
 					sa,
 				)))
+
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should accept the request when updating the clusterset of a managed cluster by authorized user", func() {
 				clusterSetName := fmt.Sprintf("webhook-spoke-%s", rand.String(6))
 				ginkgo.By(fmt.Sprintf("create a managed cluster set %q", clusterSetName))
 
-				managedClusterSet := &clusterv1alpha1.ManagedClusterSet{
+				managedClusterSet := &clusterv1beta1.ManagedClusterSet{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: clusterSetName,
 					},
 				}
 
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				sa := fmt.Sprintf("webhook-sa-%s", rand.String(6))
@@ -358,19 +381,21 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					return err
 				})
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
+
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("Should forbid the request when updating the clusterset of a managed cluster by unauthorized user", func() {
 				clusterSetName := fmt.Sprintf("webhook-spoke-%s", rand.String(6))
 				ginkgo.By(fmt.Sprintf("create a managed cluster set %q", clusterSetName))
 
-				managedClusterSet := &clusterv1alpha1.ManagedClusterSet{
+				managedClusterSet := &clusterv1beta1.ManagedClusterSet{
 					ObjectMeta: metav1.ObjectMeta{
 						Name: clusterSetName,
 					},
 				}
 
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSets().Create(context.TODO(), managedClusterSet, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				sa := fmt.Sprintf("webhook-sa-%s", rand.String(6))
@@ -406,6 +431,8 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					sa,
 					clusterSetName,
 				)))
+
+				gomega.Expect(cleanupClusterClient(saNamespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 		})
 	})
@@ -430,11 +457,11 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 			gomega.Eventually(func() bool {
 				clusterSetName := fmt.Sprintf("clusterset-%s", rand.String(6))
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetName, clusterSetName)
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				if err != nil {
 					return false
 				}
-				clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Delete(context.TODO(), clusterSetName, metav1.DeleteOptions{})
+				clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Delete(context.TODO(), clusterSetName, metav1.DeleteOptions{})
 				return true
 			}, 60*time.Second, 1*time.Second).Should(gomega.BeTrue())
 		})
@@ -448,7 +475,7 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				clusterSetName := fmt.Sprintf("clusterset-%s", rand.String(6))
 				clusterSetBindingName := fmt.Sprintf("clustersetbinding-%s", rand.String(6))
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetBindingName, clusterSetName)
-				_, err := clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				_, err := clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				gomega.Expect(err).To(gomega.HaveOccurred())
 				gomega.Expect(errors.IsBadRequest(err)).Should(gomega.BeTrue())
 				gomega.Expect(err.Error()).Should(gomega.Equal(fmt.Sprintf(
@@ -477,8 +504,10 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
 
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetName, clusterSetName)
-				_, err = authorizedClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				_, err = authorizedClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
+
+				gomega.Expect(cleanupClusterClient(namespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 
 			ginkgo.It("should forbid the request when creating a ManagedClusterSetBinding by unauthorized user", func() {
@@ -497,7 +526,7 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				gomega.Expect(err).ToNot(gomega.HaveOccurred())
 
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetName, clusterSetName)
-				_, err = unauthorizedClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				_, err = unauthorizedClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				gomega.Expect(err).To(gomega.HaveOccurred())
 				gomega.Expect(errors.IsForbidden(err)).Should(gomega.BeTrue())
 				gomega.Expect(err.Error()).Should(gomega.Equal(fmt.Sprintf(
@@ -507,6 +536,8 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 					sa,
 					clusterSetName,
 				)))
+
+				gomega.Expect(cleanupClusterClient(namespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 		})
 
@@ -515,13 +546,13 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				// create a cluster set binding
 				clusterSetName := fmt.Sprintf("clusterset-%s", rand.String(6))
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetName, clusterSetName)
-				managedClusterSetBinding, err := clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				managedClusterSetBinding, err := clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				// update the cluster set binding
 				clusterSetName = fmt.Sprintf("clusterset-%s", rand.String(6))
 				managedClusterSetBinding.Spec.ClusterSet = clusterSetName
-				_, err = clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Update(context.TODO(), managedClusterSetBinding, metav1.UpdateOptions{})
+				_, err = clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Update(context.TODO(), managedClusterSetBinding, metav1.UpdateOptions{})
 				gomega.Expect(err).To(gomega.HaveOccurred())
 				gomega.Expect(errors.IsBadRequest(err)).Should(gomega.BeTrue())
 				gomega.Expect(err.Error()).Should(gomega.Equal(fmt.Sprintf(
@@ -534,7 +565,7 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				// create a cluster set binding
 				clusterSetName := fmt.Sprintf("clusterset-%s", rand.String(6))
 				managedClusterSetBinding := newManagedClusterSetBinding(namespace, clusterSetName, clusterSetName)
-				managedClusterSetBinding, err := clusterClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
+				managedClusterSetBinding, err := clusterClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Create(context.TODO(), managedClusterSetBinding, metav1.CreateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 				// create a client without clusterset binding permission
@@ -552,8 +583,10 @@ var _ = ginkgo.Describe("Admission webhook", func() {
 				managedClusterSetBinding.Labels = map[string]string{
 					"owner": "user1",
 				}
-				_, err = unauthorizedClient.ClusterV1alpha1().ManagedClusterSetBindings(namespace).Update(context.TODO(), managedClusterSetBinding, metav1.UpdateOptions{})
+				_, err = unauthorizedClient.ClusterV1beta1().ManagedClusterSetBindings(namespace).Update(context.TODO(), managedClusterSetBinding, metav1.UpdateOptions{})
 				gomega.Expect(err).NotTo(gomega.HaveOccurred())
+
+				gomega.Expect(cleanupClusterClient(namespace, sa)).ToNot(gomega.HaveOccurred())
 			})
 		})
 	})
@@ -575,13 +608,13 @@ func newManagedCluster(name string, accepted bool, externalURL string) *clusterv
 	}
 }
 
-func newManagedClusterSetBinding(namespace, name string, clusterSet string) *clusterv1alpha1.ManagedClusterSetBinding {
-	return &clusterv1alpha1.ManagedClusterSetBinding{
+func newManagedClusterSetBinding(namespace, name string, clusterSet string) *clusterv1beta1.ManagedClusterSetBinding {
+	return &clusterv1beta1.ManagedClusterSetBinding{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: namespace,
 			Name:      name,
 		},
-		Spec: clusterv1alpha1.ManagedClusterSetBindingSpec{
+		Spec: clusterv1beta1.ManagedClusterSetBindingSpec{
 			ClusterSet: clusterSet,
 		},
 	}
@@ -715,3 +748,27 @@ func buildClusterClient(saNamespace, saName string, clusterPolicyRules, policyRu
 	})
 	return unauthorizedClusterClient, err
 }
+
+// cleanupClusterClient delete cluster-scope resource created by func "buildClusterClient",
+// the namespace-scope resources should be deleted by an additional namespace deleting func.
+// It is recommended be invoked as a pair with the func "buildClusterClient"
+func cleanupClusterClient(saNamespace, saName string) error {
+	err := hubClient.CoreV1().ServiceAccounts(saNamespace).Delete(context.TODO(), saName, metav1.DeleteOptions{})
+	if err != nil {
+		return fmt.Errorf("delete sa %q/%q failed: %v", saNamespace, saName, err)
+	}
+
+	// delete cluster role and cluster role binding if exists
+	clusterRoleName := fmt.Sprintf("%s-clusterrole", saName)
+	err = hubClient.RbacV1().ClusterRoles().Delete(context.TODO(), clusterRoleName, metav1.DeleteOptions{})
+	if err != nil && !errors.IsNotFound(err) {
+		return fmt.Errorf("delete cluster role %q failed: %v", clusterRoleName, err)
+	}
+	clusterRoleBindingName := fmt.Sprintf("%s-clusterrolebinding", saName)
+	err = hubClient.RbacV1().ClusterRoleBindings().Delete(context.TODO(), clusterRoleBindingName, metav1.DeleteOptions{})
+	if err != nil && !errors.IsNotFound(err) {
+		return fmt.Errorf("delete cluster role binding %q failed: %v", clusterRoleBindingName, err)
+	}
+
+	return nil
+}
